
/* Web Polygraph       http://www.web-polygraph.org/
 * (C) 2003-2006 The Measurement Factory
 * Licensed under the Apache License, Version 2.0 */

#include "base/polygraph.h"

#include "xstd/String.h"
#include "base/AggrStat.h"
#include "base/ContTypeStat.h"

PtrArray<String*> ContTypeStat::TheKinds;
static const String TheAnonCTypeKind = "_anonym_";
static const String TheLostCTypeKind = "_lost_";

void ContTypeStat::RecordKind(int ctype, const String &kind) {
	Assert(ctype >= 0);
	// make sure we do not miss any to simplify life
	while (TheKinds.count() < ctype)
		TheKinds.append(new String(TheAnonCTypeKind));
	if (ctype < TheKinds.count())
		delete TheKinds[ctype];
	TheKinds.put(new String(kind ? kind : TheAnonCTypeKind), ctype);
}

void ContTypeStat::Store(OLog &ol) {
	OLogStorePtrs(ol, TheKinds);
}

void ContTypeStat::Load(ILog &il) {
	if (TheKinds.empty())
		ILogLoadPtrs(il, TheKinds, String);
	else {
		PtrArray<String*> kinds;
		ILogLoadPtrs(il, kinds, String);
		bool same(TheKinds.count() == kinds.count());
		for (int i = 0; same && i < kinds.count(); ++i)
			same = *kinds[i] == *TheKinds[i];

		if (!same) {
			cerr << "error: mismatching log files (logged content "
				"types differ):" << endl <<
				"content types in the first log:" << endl;
			for (int i = 0; i < TheKinds.count(); ++i)
				cerr << (i ? ", " : "\t") << *TheKinds[i];
			cerr << endl << "content types in the last log:" << endl;
			for (int i = 0; i < kinds.count(); ++i)
				cerr << (i ? ", " : "\t") << *kinds[i];
			cerr << endl;
			exit(-2);
		}
	}
}

ContTypeStat::ContTypeStat() {
}

ContTypeStat::~ContTypeStat() {
	while (theStats.count()) delete theStats.pop();
}

void ContTypeStat::record(int ctype, Size sz) {
	// initialize storage on-demand
	if (!theStats.count()) {
		for (int i = 0; i < TheKinds.count(); ++i)
			theStats.append(new AggrStat);
	}
	Assert(0 <= ctype && ctype < theStats.count());
	theStats[ctype]->record(sz);
}

void ContTypeStat::add(const ContTypeStat &s) {
	if (!theStats.count()) {
		for (int i = 0; i < s.theStats.count(); ++i)
			theStats.append(new AggrStat(*s.theStats[i]));
	} else 
	if (s.theStats.count()) {
		Assert(theStats.count() == s.theStats.count());
		for (int i = 0; i < theStats.count(); ++i)
			*theStats[i] += *s.theStats[i];
	}
}

OLog &ContTypeStat::store(OLog &ol) const {
	OLogStorePtrs(ol, theStats);
	return ol;
}

ILog &ContTypeStat::load(ILog &il) {
	ILogLoadPtrs(il, theStats, AggrStat);
	return il;
}

ostream &ContTypeStat::print(ostream &os, const String &pfx) const {
	int count = 0;
	for (int c = 0; c < theStats.count(); ++c) {
		// handle logging bugs: buggy code may not register or log all TheKinds
		const String kind = c < TheKinds.count() ?
			*TheKinds[c] : TheLostCTypeKind;
		if (theStats[c]->count() || kind != TheAnonCTypeKind)
			count++;
	}
	os << pfx << "categories:\t " << count << endl;
	for (int i = 0; i < theStats.count(); ++i) {
		const String kind = i < TheKinds.count() ?
			*TheKinds[i] : TheLostCTypeKind;
		if (theStats[i]->count() || kind != TheAnonCTypeKind)
			theStats[i]->print(os, pfx + kind + ".size.");
	}
	return os;
}
